@@ -1,6 +1,3 @@
-# (c) 2012-2016 Continuum Analytics, Inc. / http://continuum.io
-# All Rights Reserved
-#
 # conda is distributed under the terms of the BSD 3-clause license.
 # Consult LICENSE.txt or http://opensource.org/licenses/BSD-3-Clause.
 """OS-agnostic, system-level binary package manager."""
@@ -10,7 +7,7 @@ import os
 from os.path import dirname
 import sys
 
-from .common.compat import iteritems, text_type
+from .common.compat import text_type
 
 __all__ = (
     "__name__", "__version__", "__author__", "__email__", "__license__", "__summary__", "__url__",
@@ -19,7 +16,7 @@ __all__ = (
 )
 
 __name__ = "conda"
-__version__ = "4.3.34"
+__version__ = "4.5.0"
 __author__ = "Anaconda, Inc."
 __email__ = "conda@continuum.io"
 __license__ = "BSD-3-Clause"
@@ -34,6 +31,8 @@ CONDA_PACKAGE_ROOT = dirname(__file__)
 
 
 class CondaError(Exception):
+    return_code = 1
+
     def __init__(self, message, caused_by=None, **kwargs):
         self.message = message
         self._kwargs = kwargs
@@ -46,20 +45,20 @@ class CondaError(Exception):
     def __str__(self):
         try:
             return text_type(self.message % self._kwargs)
-        except TypeError:
-            # TypeError: not enough arguments for format string
+        except:
             debug_message = "\n".join((
                 "class: " + self.__class__.__name__,
                 "message:",
                 self.message,
                 "kwargs:",
                 text_type(self._kwargs),
+                "",
             ))
-            sys.stderr.write(debug_message)
+            print(debug_message, file=sys.stderr)
             raise
 
     def dump_map(self):
-        result = dict((k, v) for k, v in iteritems(vars(self)) if not k.startswith('_'))
+        result = dict((k, v) for k, v in vars(self).items() if not k.startswith('_'))
         result.update(exception_type=text_type(type(self)),
                       exception_name=self.__class__.__name__,
                       message=text_type(self),
@@ -73,7 +72,7 @@ class CondaMultiError(CondaError):
 
     def __init__(self, errors):
         self.errors = errors
-        super(CondaError, self).__init__(None)
+        super(CondaMultiError, self).__init__(None)
 
     def __repr__(self):
         return '\n'.join(repr(e) for e in self.errors) + '\n'
@@ -93,7 +92,7 @@ class CondaMultiError(CondaError):
 
 
 class CondaExitZero(CondaError):
-    pass
+    return_code = 0
 
 
 ACTIVE_SUBPROCESSES = set()
