@@ -29,24 +29,24 @@ from enum import Enum, EnumMeta
 
 from .compat import (isiterable, iteritems, itervalues, odict, primitive_types, string_types,
                      text_type, with_metaclass)
-from .constants import EMPTY_MAP, NULL
+from .constants import NULL
 from .path import expand
-from .yaml import yaml_load
+from .serialize import yaml_load
 from .. import CondaError, CondaMultiError
 from .._vendor.auxlib.collection import AttrDict, first, frozendict, last, make_immutable
 from .._vendor.auxlib.exceptions import ThisShouldNeverHappenError
 from .._vendor.auxlib.type_coercion import TypeCoercionError, typify_data_structure
 from .._vendor.boltons.setutils import IndexedSet
 
-try:
+try:  # pragma: no cover
     from cytoolz.dicttoolz import merge
     from cytoolz.functoolz import excepts
     from cytoolz.itertoolz import concat, concatv, unique
-except ImportError:
+except ImportError:  # pragma: no cover
     from .._vendor.toolz.dicttoolz import merge
     from .._vendor.toolz.functoolz import excepts
     from .._vendor.toolz.itertoolz import concat, concatv, unique
-try:
+try:  # pragma: no cover
     from ruamel_yaml.comments import CommentedSeq, CommentedMap
     from ruamel_yaml.scanner import ScannerError
 except ImportError:  # pragma: no cover
@@ -55,6 +55,8 @@ except ImportError:  # pragma: no cover
 
 log = getLogger(__name__)
 
+EMPTY_MAP = frozendict()
+
 
 def pretty_list(iterable, padding='  '):  # TODO: move elsewhere in conda.common
     if not isiterable(iterable):
@@ -85,7 +87,7 @@ class ValidationError(ConfigurationError):
         self.parameter_name = parameter_name
         self.parameter_value = parameter_value
         self.source = source
-        super(ConfigurationError, self).__init__(msg, **kwargs)
+        super(ValidationError, self).__init__(msg, **kwargs)
 
 
 class MultipleKeysError(ValidationError):
@@ -204,14 +206,12 @@ class EnvRawParameter(RawParameter):
 
     def value(self, parameter_obj):
         if hasattr(parameter_obj, 'string_delimiter'):
+            assert isinstance(self._raw_value, string_types)
             string_delimiter = getattr(parameter_obj, 'string_delimiter')
             # TODO: add stripping of !important, !top, and !bottom
-            raw_value = self._raw_value
-            if string_delimiter in raw_value:
-                value = raw_value.split(string_delimiter)
-            else:
-                value = [raw_value]
-            return tuple(v.strip() for v in value)
+            return tuple(v for v in (
+                vv.strip() for vv in self._raw_value.split(string_delimiter)
+            ) if v)
         else:
             return self.__important_split_value[0].strip()
 
@@ -385,8 +385,9 @@ class Parameter(object):
     def _set_name(self, name):
         # this is an explicit method, and not a descriptor/setter
         # it's meant to be called by the Configuration metaclass
-        self._name = name
-        self._names = frozenset(x for x in chain(self.aliases, (name, )))
+        self._name = name  # lgtm [py/mutable-descriptor]
+        _names = frozenset(x for x in chain(self.aliases, (name, )))
+        self._names = _names  # lgtm [py/mutable-descriptor]
         return name
 
     @property
@@ -452,8 +453,8 @@ class Parameter(object):
         else:
             errors.extend(self.collect_errors(instance, result))
         raise_errors(errors)
-        instance._cache_[self.name] = result
-        return result
+        instance._cache_[self.name] = result  # lgtm [py/uninitialized-local-variable]
+        return result  # lgtm [py/uninitialized-local-variable]
 
     def collect_errors(self, instance, value, source="<<merged>>"):
         """Validate a Parameter value.
@@ -669,7 +670,7 @@ class MapParameter(Parameter):
 
         # mapkeys with important matches
         def key_is_important(match, key):
-            return match.valueflags(self).get(key) is ParameterFlag.final
+            return match.valueflags(self).get(key) == ParameterFlag.final
         important_maps = tuple(dict((k, v)
                                     for k, v in iteritems(match_value)
                                     if key_is_important(match, k))
@@ -711,54 +712,28 @@ class ConfigurationType(type):
 class Configuration(object):
 
     def __init__(self, search_path=(), app_name=None, argparse_args=None):
+        # Currently, __init__ does a **full** disk reload of all files.
+        # A future improvement would be to cache files that are already loaded.
         self.raw_data = odict()
         self._cache_ = dict()
-        self._reset_callbacks = set()  # TODO: make this a boltons ordered set
+        self._reset_callbacks = IndexedSet()
         self._validation_errors = defaultdict(list)
 
-        if not hasattr(self, '_search_path') and search_path is not None:
-            # we only set search_path once; we never change it
-            self._search_path = IndexedSet(search_path)
-
-        if not hasattr(self, '_app_name') and app_name is not None:
-            # we only set app_name once; we never change it
-            self._app_name = app_name
-
         self._set_search_path(search_path)
         self._set_env_vars(app_name)
         self._set_argparse_args(argparse_args)
 
     def _set_search_path(self, search_path):
-        if not hasattr(self, '_search_path') and search_path is not None:
-            # we only set search_path once; we never change it
-            self._search_path = IndexedSet(search_path)
-
-        if getattr(self, '_search_path', None):
-
-            # we need to make sure old data doesn't stick around if we are resetting
-            #   easiest solution is to completely clear raw_data and re-load other sources
-            #   if raw_data holds contents
-            raw_data_held_contents = bool(self.raw_data)
-            if raw_data_held_contents:
-                self.raw_data = odict()
-
-            self._set_raw_data(load_file_configs(search_path))
-
-            if raw_data_held_contents:
-                # this should only be triggered on re-initialization / reset
-                self._set_env_vars(getattr(self, '_app_name', None))
-                self._set_argparse_args(self._argparse_args)
-
+        self._search_path = IndexedSet(search_path)
+        self._set_raw_data(load_file_configs(search_path))
         self._reset_cache()
         return self
 
     def _set_env_vars(self, app_name=None):
-        if not hasattr(self, '_app_name') and app_name is not None:
-            # we only set app_name once; we never change it
-            self._app_name = app_name
-        if getattr(self, '_app_name', None):
-            erp = EnvRawParameter
-            self.raw_data[erp.source] = erp.make_raw_parameters(self._app_name)
+        self._app_name = app_name
+        if not app_name:
+            return self
+        self.raw_data[EnvRawParameter.source] = EnvRawParameter.make_raw_parameters(app_name)
         self._reset_cache()
         return self
 
@@ -812,8 +787,13 @@ class Configuration(object):
 
             if match is not None:
                 try:
-                    typed_value = typify_data_structure(match.value(parameter),
-                                                        parameter._element_type)
+                    untyped_value = match.value(parameter)
+                    if untyped_value is None:
+                        if isinstance(parameter, SequenceParameter):
+                            untyped_value = ()
+                        elif isinstance(parameter, MapParameter):
+                            untyped_value = {}
+                    typed_value = typify_data_structure(untyped_value, parameter._element_type)
                 except TypeCoercionError as e:
                     validation_errors.append(CustomValidationError(match.key, e.value,
                                                                    match.source, text_type(e)))
