@@ -3,14 +3,18 @@ from __future__ import absolute_import, division, print_function, unicode_litera
 
 from errno import EINVAL, EXDEV
 from logging import getLogger
+import os
 from os import rename as os_rename, utime
-from os.path import dirname, isdir, lexists
+from os.path import dirname, isdir
 import re
+from shutil import move
 
-from . import exp_backoff_fn
+from . import exp_backoff_fn, mkdir_p, mkdir_p_sudo_safe
 from .delete import rm_rf
-from .link import islink
+from .link import lexists
+from ...common.compat import on_win
 from ...common.path import expand
+from ...exceptions import NotWritableError
 
 log = getLogger(__name__)
 
@@ -41,15 +45,6 @@ def update_file_in_place_as_binary(file_full_path, callback):
             fh.close()
 
 
-def _copy_then_remove(source_path, destination_path):
-    from .create import copy, create_hard_link_or_copy
-    if islink(source_path):
-        copy(source_path, destination_path)
-    else:
-        create_hard_link_or_copy(source_path, destination_path)
-    rm_rf(source_path)
-
-
 def rename(source_path, destination_path, force=False):
     if lexists(destination_path) and force:
         rm_rf(destination_path)
@@ -59,10 +54,13 @@ def rename(source_path, destination_path, force=False):
             os_rename(source_path, destination_path)
         except EnvironmentError as e:
             if e.errno in (EINVAL, EXDEV):
-                # see https://github.com/conda/conda/issues/6711
-                log.trace("Could not rename do to errno [%s]. Falling back to copy/remove.",
-                          e.errno)
-                _copy_then_remove(source_path, destination_path)
+                # https://github.com/conda/conda/issues/6811
+                # https://github.com/conda/conda/issues/6711
+                log.trace("Could not rename %s => %s due to errno [%s]. Falling back"
+                          " to copy/unlink", source_path, destination_path, e.errno)
+                # https://github.com/moby/moby/issues/25409#issuecomment-238537855
+                # shutil.move() falls back to copy+unlink
+                move(source_path, destination_path)
             else:
                 raise
     else:
@@ -73,22 +71,38 @@ def backoff_rename(source_path, destination_path, force=False):
     exp_backoff_fn(rename, source_path, destination_path, force)
 
 
-def touch(path):
-    # returns
+def touch(path, mkdir=False, sudo_safe=False):
+    # sudo_safe: use any time `path` is within the user's home directory
+    # returns:
     #   True if the file did not exist but was created
     #   False if the file already existed
-    # raises permissions errors such as EPERM and EACCES
-    path = expand(path)
-    log.trace("touching path %s", path)
-    if lexists(path):
-        utime(path, None)
-        return True
-    else:
-        assert isdir(dirname(path))
-        try:
-            fh = open(path, 'a')
-        except:
-            raise
+    # raises: permissions errors such as EPERM and EACCES
+    try:
+        path = expand(path)
+        log.trace("touching path %s", path)
+        if lexists(path):
+            utime(path, None)
+            return True
         else:
-            fh.close()
-            return False
+            dirpath = dirname(path)
+            if not isdir(dirpath) and mkdir:
+                if sudo_safe:
+                    mkdir_p_sudo_safe(dirpath)
+                else:
+                    mkdir_p(dirpath)
+            else:
+                assert isdir(dirname(path))
+            try:
+                fh = open(path, 'a')
+            except:
+                raise
+            else:
+                fh.close()
+                if sudo_safe and not on_win and os.environ.get('SUDO_UID') is not None:
+                    uid = int(os.environ['SUDO_UID'])
+                    gid = int(os.environ.get('SUDO_GID', -1))
+                    log.trace("chowning %s:%s %s", uid, gid, path)
+                    os.chown(path, uid, gid)
+                return False
+    except (IOError, OSError) as e:
+        raise NotWritableError(path, e.errno, caused_by=e)
